%{
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "structs.h"
#include "y.tab.h"

extern YYSTYPE yylval;
int lineNo = 1;


%}

punctuation	"("|"["|"{"|")"|"]"|"}"|","|":"|";"|"."
ops 	"+"|"-"|"*"|"/"|"^"|"<"|">"|"="|"#"|"|"|"~"|"&"|"@"

relOp		"<>"|"<="|">="

comment         \(\*([^*]|[\r\n]|(\*+([^*)]|[\r\n])))*\*+\)

newline		[\n]
whitespace	[ \t]
character1	[^""\n"]
character2 	[^'"\n"]

octalDigit	[0-7]
digit		[0-9]
hexDigit	[0-9A-F]
ScaleFactor	"K"[-+]?{digit}+
real		{digit}+"."{digit}+{ScaleFactor}?
integer		{digit}+|{octalDigit}+[BC]|{hexDigit}+[H]
number		{integer}|{real}
		
ident		[a-zA-Z][a-zA-Z0-9]*
		
string		([\"]{character1}*[\"])|('{character2}*')


%%
{comment}	{
              	//printf("comment ");
		int i;
		for(i=0;i<yyleng;i++){
			if(yytext[i]=='\n')lineNo++;
		}
               	}
{punctuation}	{//printf(" punctuation(%s) ", yytext); 
		
		yylval.str =(char *) malloc(yyleng*sizeof(char));
		strcpy(yylval.str, yytext);
		return *yytext;
		}

		
{newline}	{
		lineNo++;
		//printf("\n%d : ",lineNo); 
		 
		}

{whitespace}+	{
		//printf(" ");
		}

{ops}		{
		//printf("ops(%s) ",yytext);
		return *yytext;
		}
		
"<="		{
		//printf("relOp(%s) ",yytext); 
		yylval.str =(char *) malloc(yyleng*sizeof(char));
		strcpy(yylval.str, yytext);
			return LEQ;
		}
">="		{
		yylval.str =(char *) malloc(yyleng*sizeof(char));
		strcpy(yylval.str, yytext);
			return GEQ;
		}
"<>"		{
		yylval.str =(char *) malloc(yyleng*sizeof(char));
		strcpy(yylval.str, yytext);
			return NEQ;
		}
BOOLEAN		{
			return BOOLEAN;
		}
CHAR		{	
			return CHAR;
		}
REAL		{	
			return REAL;
		}
CARDINAL	{
			return CARDINAL;
		}
INTEGER		{
		
			return INTEGER;
		}

":="		{
		//printf("ASSIGNMENT "); 
		return ASS_OP;
		}

".."		{
		//printf("RANGE %s",yytext); 
		return ELLIPSIS;
		}

"AND"		{
		//printf("AND "); 
		yylval.str =(char *) malloc(yyleng*sizeof(char));
		strcpy(yylval.str, yytext);
		return AND;
		}
		
"FOR"		{
		//printf("%s ",yytext); 
		return FOR;
		}
		
"FINALLY"	{
		//printf("%s ",yytext); 
		return FINALLY;
		}       
		 
"PROCEDURE"	{
		//printf("%s ",yytext); 
		return PROCEDURE;
		}	
"EXPORT"        {
		//printf("%s ",yytext); 
		return EXPORT;
		}
		
"POINTER"	{
		//printf("%s ",yytext); 
		return POINTER;
		}
		
"EXIT"		{
		//printf("%s ",yytext); 
		return EXIT;
		}    
		       
"PACKEDSET"   	{
		//printf("%s ",yytext); 
		return PACKEDSET;
		}   
		               
"WITH"		{
		//printf("%s ",yytext); 
		return WITH;
		}
		        
"EXCEPT"      	{
		//printf("%s ",yytext); 
		return EXCEPT;
		}
		              
"OR"           	{
		yylval.str =(char *) malloc(yyleng*sizeof(char));
		strcpy(yylval.str, yytext);
		//printf("%s ",yytext); 
		return OR;
		}
		                 
WHILE		{
		//printf("%s ",yytext); 
		return WHILE;
		}
		           
"END"          	{
		//printf("%s ",yytext); 
		return END;
		}
		             
"OF"        	{
		//printf("%s ",yytext); 
		return OF;
		}
		                    
"VAR"		{
		//printf("%s ",yytext); 
		return VAR;
		}
		           
"ELSIF"       	{
		//printf("%s ",yytext); 
		return ELSIF;
		}
		              
"NOT"        	{
		//printf("%s ",yytext); 
		return NOT;
		}
		                   
"UNTIL"		{
		//printf("%s ",yytext); 
		return UNTIL;
		}
		           
"ELSE"    	{
		//printf("%s ",yytext); 
		return ELSE;
		} 
		                 
"MODULE"	{
		//printf("%s ",yytext); 
		return MODULE;
		}
		                        
"TYPE"		{
		//printf("%s ",yytext); 
		return TYPE;
		}
		           
"QUALIFIED"	{
		//printf("%s ",yytext); 
		return QUALIFIED;
		}
		           
"ARRAY"         {
		//printf("%s ",yytext); 
		return ARRAY;
		} 
		          
"FORWARD"           	{
		//printf("%s ",yytext); 
		return FORWARD;
		}
		         
"RECORD"	{
		//printf("%s ",yytext); 
		return RECORD;
		}    

		      
"FROM"          {
		//printf("%s ",yytext); 
		return FROM;
		}           
"REPEAT"	{
		//printf("%s ",yytext); 
		return REPEAT;
		}  
		         
"BY"            {
		//printf("%s ",yytext); 
		return BY;
		} 
		          
"IF"             {
		//printf("%s ",yytext); 
		return IF;
		} 
		          
"REM"	{
		//printf("%s ",yytext); 
		return REM;
		} 
		          
"CASE"           	{
		//printf("%s ",yytext); 
		return CASE;
		} 
		          
"IMPLEMENTATION"     	{
		//printf("%s ",yytext); 
		return IMPLEMENTATION;
		}   
		        
"RETRY"		{
		//printf("%s ",yytext); 
		return RETRY;
		}    
		       
"CONST"          	{
		//printf("%s ",yytext); 
		return CONST;
		}   
		        
"IMPORT"             	{
		//printf("%s ",yytext); 
		return IMPORT;
		}  
		         
"RETURN"	{
		//printf("%s ",yytext); 
		return RETURN;
		}    
		       
"DEFINITION"     	{
		//printf("%s ",yytext); 
		return DEFINITION;
		}   
		        
"IN"                 	{
		//printf("%s ",yytext); 
		return IN;
		}
		           
"SET"		{
		//printf("%s ",yytext); 
		return SET;
		}
		           
"DIV"            	{
		//printf("%s ",yytext); 
		yylval.str =(char *) malloc(yyleng*sizeof(char));
		strcpy(yylval.str, yytext);
		return DIV;
		} 
		          
"LOOP"               	{
		//printf("%s ",yytext); 
		return LOOP;
		}
		           
"THEN"		{
		//printf("%s ",yytext); 
		return THEN;
		}   
		        
"PRINT"             	{
		//printf("%s ",yytext); 
		return PRINT;
		}           
"DO"             	{
		//printf("%s ",yytext); 
		return DO;
		}           
"MOD"                	{
		//printf("%s ",yytext); 
		yylval.str =(char *) malloc(yyleng*sizeof(char));
		strcpy(yylval.str, yytext);
		return MOD;
		}    
		       
"TO"		{//printf("%s ",yytext); 
		return TO;
		}


"BEGIN"         {
			//printf("%s ",yytext); 
			return BEG;
		}     


"VAL"         {
	//printf("%s ",yytext); 
	return VAL;
} 	
		
{integer} 	{
		//printf("whole_number_literal(%s) ", yytext);
		yylval.str =(char *) malloc(yyleng*sizeof(char));
		strcpy(yylval.str, yytext);
		return INTEGER_LIT;
		}

{real} 		{
		//printf("real_literal(%s) ", yytext);
		yylval.str =(char *) malloc(yyleng*sizeof(char));
		strcpy(yylval.str, yytext);
		return REAL_LIT;
		}

{ident}		{
		yylval.str=(char*)malloc(yyleng*sizeof(char));
		strcpy(yylval.str, yytext);
		//printf("identifier");		
		return IDENTIFIER;
		}
{string}	{
		//printf("string_literal(%s) ", yytext);
		yylval.str =(char *) malloc(yyleng*sizeof(char));
		strcpy(yylval.str, yytext);
		return STRING_LIT;
		}

.	;
%%
int yywrap(){
	return 1;
}
/*int main()
{	yylex();
	return 0;
}
*/
